#!/usr/bin/env python

# stdlib imports
import os.path
import sys
import tempfile

# hack the path so that I can debug these functions if I need to
homedir = os.path.dirname(os.path.abspath(__file__))  # where is this script?
impactdir = os.path.abspath(os.path.join(homedir, "..", ".."))
# put this at the front of the system path, ignoring any installed mapio stuff
sys.path.insert(0, impactdir)

from esi_utils_geo.city import Cities


def test():
    print("Test loading geonames cities file...")
    cities = Cities.fromDefault()  # load from a file contained in repo
    dataframe = cities.getDataFrame()
    idx = (dataframe["name"] == "Los Angeles") & (dataframe["ccode"] == "US")
    assert len(dataframe.loc[idx, :]) == 1
    print("Passed loading geonames cities file.")

    print("Test getting city names and coordinates...")
    lat, _, _ = cities.getCities()
    assert len(lat) == len(cities)
    print("Passed getting city names and coordinates...")

    print("Test limiting cities using California bounds...")
    ymin, ymax = 32.394, 42.062
    xmin, xmax = -125.032, -114.002
    bcities = cities.limitByBounds((xmin, xmax, ymin, ymax))
    bounds = bcities.getBounds()
    assert (
        bounds[0] > xmin and bounds[1] < xmax and bounds[2] > ymin and bounds[3] < ymax
    )
    print("Passed limiting cities using California bounds.")

    print("Test limiting cities using a 4 by 4 grid...")
    gcities = bcities.limitByGrid(nx=2, ny=4, cities_per_grid=10)
    assert len(gcities) <= 2 * 4 * 10
    print("Passed limiting cities using California bounds.")

    print("Test getting cities by name (Los Angeles)...")
    cityofangels = bcities.limitByName("Los Angeles").limitByPopulation(1000000)
    assert len(cityofangels) == 1
    print("Passed getting cities by name (Los Angeles).")

    print("Test limiting cities 50 km radius around LA...")
    clat, clon = 34.048019, -118.244133
    rcities = bcities.limitByRadius(clat, clon, 50)
    print("Passed limiting cities using California bounds.")

    print("Test limiting cities above population above 50,000...")
    popthresh = 50000
    bigcities = rcities.limitByPopulation(popthresh)
    df = bigcities.getDataFrame()
    assert df["pop"].max() >= popthresh
    print("Test limiting cities above population above 50,000.")

    print("Test saving cities and reading them back in...")
    foo, tmpfile = tempfile.mkstemp()
    os.close(foo)
    bigcities.save(tmpfile)
    newcities = Cities.fromCSV(tmpfile)
    assert len(bigcities) == len(newcities)
    print("Passed saving cities and reading them back in.")

    print("Test sorting by columns")


def test_bounds_meridian():
    bounds = (174.35, -175.8972, -41.2321, -33.5167)
    cities = Cities.fromDefault()
    newcities = cities.limitByBounds(bounds)
    assert len(newcities) == 291


if __name__ == "__main__":
    test_bounds_meridian()
    test()
